# -*- coding: utf-8 -*-
from pathlib import Path

from appdirs import site_data_dir
from loguru import logger

from cjk_commons.settings import get_path_attribute
from commons_1c.version import get_version_as_number

logger.disable(__name__)


def get_last_exe_file_fullpath(file_name: str, **kwargs) -> Path:
    result = None

    config_file_fullpath = get_path_attribute(
        kwargs, 'config_file_path', default_path=Path(site_data_dir('1CEStart', '1C'), '1CEStart.cfg'), is_dir=False,
        check_if_exists=False)

    if config_file_fullpath.is_file():
        installed_location_fullpaths = []

        with config_file_fullpath.open(encoding='utf-16') as config_file:
            for line in config_file.readlines():
                key_and_value = line.split('=')
                if key_and_value[0] == 'InstalledLocation':
                    value = '='.join(key_and_value[1:])
                    installed_location_fullpaths.append(Path(value.rstrip('\n')))

        platform_versions = []

        for installed_location_fullpath in installed_location_fullpaths:
            if installed_location_fullpath.is_dir():
                for version_dir_fullpath in installed_location_fullpath.rglob('*'):  # todo
                    version_as_number = get_version_as_number(version_dir_fullpath.name)
                    if version_as_number:
                        exe_file_fullpath = Path(version_dir_fullpath, 'bin', file_name)
                        if exe_file_fullpath.is_file():
                            platform_versions.append((version_as_number, exe_file_fullpath))

        platform_versions_reversed = sorted(platform_versions, key=lambda x: x[0], reverse=True)

        if platform_versions_reversed:
            result = platform_versions_reversed[0][1]
    else:
        raise FileExistsError('1CEStart.cfg file does not exist')

    return result


def get_last_1c_exe_file_fullpath(**kwargs) -> Path:
    return get_last_exe_file_fullpath('1cv8.exe', **kwargs)


def get_last_ibcmd_exe_file_fullpath(**kwargs) -> Path:
    return get_last_exe_file_fullpath('ibcmd.exe', **kwargs)


def get_last_rac_exe_file_fullpath(**kwargs) -> Path:
    return get_last_exe_file_fullpath('rac.exe', **kwargs)



# -*- coding: utf-8 -*-
from typing import List

from loguru import logger
import re

logger.disable(__name__)

pattern_version = re.compile(r'\D*(?P<version>(?:(\d+)|)(?:\.(\d+)|)(?:\.(\d+)|)(?:\.(\d+)|))\D*')


def get_version_as_number(version: str, m: int = 10000) -> int:
    result = 0

    match = pattern_version.match(version)
    if match is not None:
        a = match.group(2)
        a = '0' if a is None else a
        b = match.group(3)
        b = '0' if b is None else b
        c = match.group(4)
        c = '0' if c is None else c
        d = match.group(5)
        d = '0' if d is None else d
        result = int(a) * m ** 3 + int(b) * m ** 2 + int(c) * m + int(d)

    return result


def get_version_as_parts(version: str) -> List[str]:
    result = []
    match = pattern_version.match(version)
    if match is not None:
        a = match.group(2)
        if a is not None:
            result.append(a)
        b = match.group(3)
        if b is not None:
            result.append(b)
        c = match.group(4)
        if c is not None:
            result.append(c)
        d = match.group(5)
        if d is not None:
            result.append(d)
    return result



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['commons_1c']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0', 'cjk-commons>=3.6.2,<4.0.0', 'loguru>=0.6.0,<0.7.0']

setup_kwargs = {
    'name': 'commons-1c',
    'version': '3.6.2',
    'description': 'Commons for 1C:Enterprise',
    'long_description': 'Commons for 1C:Enterprise\n===\n',
    'author': 'Cujoko',
    'author_email': 'cujoko@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Cujoko/commons-1c',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
